-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD: Digital Circuits and Systems
--------------------------------------------------------------------------------
-- P7 example tutorial on prototyping: Counter_mod60_top
-- Plan C2 on using components

-- Target board: Legacy JK_HWD4128V board 
-- Target CPLD:  LC4128V-75TN100C
-- https://digsys.upc.edu/csd/chips/legacy/legacy.html#Lattice


-- This file is a translation of schematics at: 
-- https://digsys.upc.edu/csd/chips/legacy/JK_HWD/Counter_BCD_mod60.html
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
			
ENTITY Counter_BCD_mod60_top IS
	PORT(	
		CD_L			: 	IN  std_logic; -- Reset (clear direct)
		OSC_CLK_in		: 	IN  std_logic; -- Board's crystal oscillator
		CE				: 	IN  std_logic; -- Count enable		
		UD_L			: 	IN  std_logic; -- Up or down counting
		CPB_L			: 	IN  std_logic; -- user generation of CLK edges 
		Sel_pulse		: 	IN  std_logic; -- Select manual pulses for CLK		
		Sel_filter		: 	IN  std_logic; -- Select the debouncing filter	
		Sel_CLK_freq	: 	IN  std_logic; -- Select two frequencies for CLK

		Ta, Tb, Tc		:	OUT std_logic; -- Tens 7-segment digit
		Td, Te, Tf, Tg	:	OUT std_logic; 
		
		Ua, Ub, Uc		:	OUT std_logic; -- Units 7-segment digit
		Ud, Ue, Uf, Ug	:	OUT std_logic; 
	
		TC60			:	OUT std_logic; -- Terminal count after 60 counts
		LED_CLK_1Hz_SQ	:	OUT std_logic; -- LED to indicate 1 Hz 
		LU				:	OUT std_logic_vector(3 downto 0); -- LED to indicate Units 		
		LT				:	OUT std_logic_vector(3 downto 0) -- LED to indicate Tens		
		
	);
END Counter_BCD_mod60_top;

--------------------------------------------------------------------------------
	-- The description of the proposed schematic
	-- 
ARCHITECTURE schematic_planC2 OF Counter_BCD_mod60_top IS

	COMPONENT Counter_BCD_mod60 IS 
	PORT (
		UD_L, CE, CD, CLK	: IN	STD_LOGIC;
		D_Tens, D_Units		: OUT	STD_LOGIC_VECTOR (3 DOWNTO 0);
		TC60				: OUT	STD_LOGIC
	 );	
	END COMPONENT;   
	
	COMPONENT Hex_7seg_decoder IS  
	PORT (
		X_IN   					: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
		BI_L   					: IN STD_LOGIC;      
		A, B, C, D, E, F, G  	: OUT STD_LOGIC	
		);
	end COMPONENT; 

	COMPONENT CLK_Generator IS  
      PORT(	
			OSC_CLK_in	:	IN	STD_LOGIC;
			CD			:	IN	STD_LOGIC;
			CE			:	IN	STD_LOGIC;
			CLK_A_SQ	:	OUT	STD_LOGIC; 	-- 100 Hz squared
			CLK_B_SQ	:	OUT	STD_LOGIC; 	-- 20 Hz squared
			CLK_C_SQ	:	OUT	STD_LOGIC	-- 1 Hz squared	
		);
	end COMPONENT;       
	
	COMPONENT MUX_2 IS  
		PORT (
			Ch0, Ch1	: 	IN 	STD_LOGIC; 
			S			:	IN 	STD_LOGIC;	
			E			: 	IN 	STD_LOGIC;
			Y			: 	OUT	STD_LOGIC
		  );
	end COMPONENT; 

	COMPONENT debouncing_filter IS
		Port (	CLK 	: IN	STD_LOGIC;
				PB_L	: IN	STD_LOGIC;
				CD   	: IN 	STD_LOGIC;
				QA 		: OUT	STD_LOGIC;
				QB		: OUT	STD_LOGIC
		);
	END COMPONENT;
	

-- Signals for connecting components together (the wires)
	SIGNAL CD, CLK			: std_logic;
	SIGNAL CLK_100Hz_SQ		: std_logic;	
	SIGNAL CLK_20Hz_SQ		: std_logic;
	SIGNAL CLK_1Hz_SQ		: std_logic;	
	Signal DU, DT			: STD_LOGIC_VECTOR(3 downto 0); 
	Signal CLK_PB, CLK_wave, CLK_pulse	: STD_LOGIC;
	Signal QA, QB						: STD_LOGIC;
	
--------------------------------------------------------------------------------
BEGIN  
-- Instantiation of components
	Chip1 : Counter_BCD_mod60        
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK		=> CLK,
			CD		=> CD,
			CE		=> CE,
			UD_L	=> UD_L,
			D_tens	=> DT,
			D_units	=> DU,			
			TC60	=> TC60
		);
   
	Chip2 : CLK_Generator        
-- 	from component name => to signal or port name	
		PORT MAP (
			CD			=> CD,
			CE			=> '1',
			OSC_CLK_in	=> OSC_CLK_in,
			CLK_A_SQ	=> CLK_100Hz_SQ, 		
			CLK_B_SQ	=> CLK_20Hz_SQ, 
			CLK_C_SQ	=> CLK_1Hz_SQ
		);     
		
	Chip3 : MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			E			=> '1',
			S			=> Sel_pulse,
			Ch0			=> CLK_wave, 
			Ch1	   		=> CLK_pulse, 			
			Y          	=> CLK
		);  


	Chip4 : MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			E			=> '1',
			S			=> Sel_CLK_freq,
			Ch0			=> CLK_20Hz_SQ, 
			Ch1	   		=> CLK_1Hz_SQ, 			
			Y          	=> CLK_wave
		);   
	
	Chip5 : debouncing_filter
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK			=> CLK_100Hz_SQ,
			PB_L		=> CPB_L,
			CD			=> CD, 
			QA			=> QA, 			
			QB			=> QB
		); 	
 
	Chip6 : MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			E			=> '1',
			S			=> Sel_filter,
			Ch0			=> CLK_PB, 
			Ch1	   		=> QB, 			
			Y          	=> CLK_pulse
		);  

	Chip7 : Hex_7seg_decoder
-- 	from component name => to signal or port name	
		PORT MAP (
			X_IN		=> DT,
			BI_L		=> '1',
			a			=> Ta, 
			b	   		=> Tb, 			
			c        	=> Tc,
			d			=> Td, 
			e	   		=> Te, 			
			f        	=> Tf,		
			g			=> Tg			
		);  
	Chip8 : Hex_7seg_decoder
-- 	from component name => to signal or port name	
		PORT MAP (
			X_IN		=> DU,
			BI_L		=> '1',
			a			=> Ua, 
			b	   		=> Ub, 			
			c        	=> Uc,
			d			=> Ud, 
			e	   		=> Ue, 			
			f        	=> Uf,		
			g			=> Ug			
		);  
 
-- Buffers and logic to adapt the board hardware
CD 			<= not (CD_L); 	-- The board has active-low push-buttons
CLK_PB		<=	not(CPB_L); -- active high raw external pulse


LT	<=	DT; 	-- LED active-high
LU	<=	DU; 	-- LED active-high
LED_CLK_1Hz_SQ	<=	CLK_1Hz_SQ; 	-- LED active-high

--------------------------------------------------------------------------------
END schematic_planC2;






